/*
 *  ======== main.c ========
 */

#include <stdio.h>
#include <xdc/std.h>
#include <xdc/runtime/Error.h>
#include <xdc/runtime/System.h>
#include <ti/sysbios/BIOS.h>
#include <ti/sysbios/knl/Task.h>

#include <ti/ndk/inc/netmain.h>

#include <ti/platform/platform.h>
#include "ti/platform/resource_mgr.h"
// Our NETCTRL callback functions

static void   NetworkOpen();
static void   NetworkClose();
static void   NetworkIPAddr( IPN IPAddr, uint IfIdx, uint fAdd );
int dtask_udp_hello( SOCKET s, UINT32 unused );

void taskFxn(UArg a0, UArg a1);
void NDK_task_buile();


//---------------------------------------------------------------------------
// Configuration
//
char *HostName    = "tidsp";
char *LocalIPAddr = "192.168.2.100";
char *LocalIPMask = "255.255.255.0";    // Not used when using DHCP
char *GatewayIP   = "192.168.2.101";    // Not used when using DHCP
char *DomainName  = "demo.net";         // Not used when using DHCP
char *DNSServer   = "0.0.0.0";          // Used when set to anything but zero

Uint8 clientMACAddress [6] = {0x5C, 0x26, 0x0A, 0x69, 0x44, 0x0B}; /* MAC address for my PC */

HANDLE hHello=0;
/*
 *  ======== main ========
 */
void main()
{ 
    Task_Handle task;
    Error_Block eb;
    System_printf("enter main()\n");
    Error_init(&eb);
    task = Task_create(taskFxn, NULL, &eb);
    if (task == NULL)
    {
        System_printf("Task_create() failed!\n");
        BIOS_exit(0);
    }

    BIOS_start();     /* enable interrupts and start SYS/BIOS */
}

/*******************************************************
 * taskFxn
 *     UArg a0, UArg a1
 *     ܣ½
 * ֵ
 *******************************************************/
void taskFxn(UArg a0, UArg a1)
{
    Task_Handle NDK_task;
    Task_Params NDK_Params;
    Error_Block eb;
    Error_init(&eb);
    Task_Params_init(&NDK_Params);
    NDK_Params.stackSize = 512; //ջ512
    NDK_Params.priority  = 15;  //ȼ15
    NDK_task=Task_create(NDK_task_buile,&NDK_Params,&eb);
    if(NDK_task==NULL)
    {
      System_printf("Task_create() failed!\n");
      BIOS_exit(0);
    }

	for(;;)
	{
    Task_sleep(10);
	}
}
/*******************************************************
 * NDK_task_buile
 *     
 *     ܣ
 * ֵ
 *******************************************************/
void NDK_task_buile()
{

    for(;;)
   {
    int rc;
    CI_IPNET NA;
    CI_ROUTE RT;
    HANDLE hcfg;
    /**************************ϵͳ********************/
    rc = NC_SystemOpen( NC_PRIORITY_LOW, NC_OPMODE_INTERRUPT );
    if( rc )
    {
    	printf("NC_SystemOpen Failed (%d)\n",rc);
    	goto main_exit;
    }
    /********************һµ*********************/
    hcfg=CfgNew();
    if(!hcfg)
    {
    	printf("Create configuration Failed (%d)\n");
    	goto main_exit;
    }
    /********************ͿͻǷ*********************/
    if(strlen(DomainName)>=CFG_DOMAIN_MAX||strlen(HostName)>=CFG_HOSTNAME_MAX)
    {
    	printf("Names too long\n");
    	goto main_exit;
    }
    /********************IPַ*********************/
    bzero(&NA,sizeof(NA));
    NA.IPAddr=inet_addr(LocalIPAddr);
    NA.IPMask=inet_addr(LocalIPMask);
    strcpy(NA.Domain,DomainName);
    NA.NetType=0;
    /********************õ1*********************/
    CfgAddEntry(hcfg,CFGTAG_IPNET,1,0,sizeof(CI_IPNET),(UINT8 *)&NA,0);
    /********************hostnameӿ1*********************/
    CfgAddEntry(hcfg,CFGTAG_SYSINFO,CFGITEM_DHCP_HOSTNAME,0,strlen(HostName),(UINT8 *)&HostName,0);

    bzero(&RT,sizeof(RT));
    RT.IPDestAddr=0;
    RT.IPDestMask=0;
    RT.IPGateAddr=inet_addr(GatewayIP);
    /********************route*********************/
    CfgAddEntry(hcfg,CFGTAG_ROUTE,0,0,sizeof(CI_ROUTE),(UINT8 *)&RT,0);
    do
    {
        rc = NC_NetStart( hcfg, NetworkOpen, NetworkClose, NetworkIPAddr );
    } while( rc > 0 );
    Task_sleep(10);

    CfgFree( hcfg );

    // Close the OS
main_exit:
    NC_SystemClose();

    }

}


static void NetworkOpen()
{
	hHello = DaemonNew( SOCK_DGRAM, 0, 7, dtask_udp_hello,OS_TASKPRINORM, OS_TASKSTKNORM, 0, 1 );
}

static void NetworkClose()
{
	DaemonFree( hHello );
}

static void NetworkIPAddr( IPN IPAddr, uint IfIdx, uint fAdd )
{
    IPN IPTmp;
   /*
    if( fAdd )
        printf("Network Added: ");
    else
        printf("Network Removed: ");

    // Print a message
    IPTmp = ntohl( IPAddr );
    printf("If-%d:%d.%d.%d.%d\n", IfIdx,
            (UINT8)(IPTmp>>24)&0xFF, (UINT8)(IPTmp>>16)&0xFF,
            (UINT8)(IPTmp>>8)&0xFF, (UINT8)IPTmp&0xFF );*/
}

int dtask_udp_hello( SOCKET s, UINT32 unused )
{
   /* struct sockaddr_in sin1;
    struct timeval     to;
    int                i,tmp;
    char               *pBuf;
    HANDLE             hBuffer;

    (void)unused;

    // Configure our socket timeout to be 3 seconds
    to.tv_sec  = 3;
    to.tv_usec = 0;
    setsockopt( s, SOL_SOCKET, SO_SNDTIMEO, &to, sizeof( to ) );
    setsockopt( s, SOL_SOCKET, SO_RCVTIMEO, &to, sizeof( to ) );

    for(;;)
    {
        tmp = sizeof( sin1 );
        i = (int)recvncfrom( s, (void **)&pBuf, 0, (PSA)&sin1, &tmp, &hBuffer );

        // Spit any data back out
        if( i >= 0 )
        {
            sendto( s, pBuf, i, 0, (PSA)&sin1, sizeof(sin1) );
            recvncfree( hBuffer );
        }
        else
            break;
    }

    // Since the socket is still open, return "1"
    // (we need to leave UDP sockets open)
    */
    return(1);
}




